#!/bin/bash
# Backup firmware on a rooted Android 2.2.1 on Samsung Galaxy S (probably others
# too) over USB without flashing! For rooting use the script `doroot.sh`.
# Based on idea of maniac3389 for "Roto Backup"
# (from http://forum.xda-developers.com/showthread.php?t=850359)
#
# Usage:
#  - boot into Linux, connect over USB in USB debugging mode and get ADB command
#    to work (./bin/adblinux)
#  - root it: ./doroot.sh
#  - do backup: ./dobackup.sh
#  (In case you want to give the generated image to somebody else it is
#  recommended to do a factory reset. You may also modify this scripts to suite
#  your environment and needs (eg. usage of tar).)
#
# Restoration is meant to be done with heimdall:
#  adb push data.tar.gz /sdcard/data.tar.gz
#  adb shell su -c 'tar -zxvf /sdcard/data.tar.gz'
#  adb reboot download
#  heimdall flash --kernel zImage --factoryfs factoryfs.rfs --modem modem.bin --cache cache.rfs --dbdata dbdata.rfs --param param.lfs
# But you can also create a flashable Odin tar:
#  tar --format=ustar -cf PDA.tar boot.bin cache.rfs dbdata.rfs factoryfs.rfs modem.bin param.lfs Sbl.bin zImage dbdata.rfs
#  Load up the PDA.tar into the PDA slot in Odin, load up the pit in the pit slot, flash...
#
# Note:
#  Before restoring/flashing check if you can get into the Download mode
#  using the 3-buttom combination (volume down + home + power on).
# Note:
#  This will create a full image of everything, also boot partitions that
#  are dangerous to flash (can brick so that not even download mode works).
# Warning:
#  Use at your own risk!
#
# Copyright (C) 2011 GW <gw.2011@tnode.com or http://gw.tnode.com/>
# License: GNU GPLv3
# URL: http://gw.tnode.com/0469-Android/f/
# Date: $HgDate: 2011/02/08 23:35:54 $

CMD='./bin/adblinux'
DST="img-${1:-`date +%y%m%d-%H%M`}"

echo 'Backup firmware on a rooted Android 2.2.1 on Samsung Galaxy S over USB without flashing!'
echo
echo 'Remember to enable USB debugging mode (Settings/Applications/Development mode/USB debugging) on the phone and connect it over USB.'
echo "Destination: $DST"
mkdir "$DST"
echo
sleep 10

echo 'Making images on the internal SD card...'
echo '(This may take a while, go make a pizza or something. :>)'
echo
echo 'dd if=/dev/block/bml1 of=/sdcard/boot.bin bs=4096'
$CMD shell su -c 'dd if=/dev/block/bml1 of=/sdcard/boot.bin bs=4096'
echo 'dd if=/dev/block/bml2 of=/sdcard/pit.pit bs=4096'
$CMD shell su -c 'dd if=/dev/block/bml2 of=/sdcard/pit.pit bs=4096'
echo 'dd if=/dev/block/bml3 of=/sdcard/untested-efs.rfs bs=4096'
$CMD shell su -c 'dd if=/dev/block/bml3 of=/sdcard/untested-efs.rfs bs=4096'
echo 'dd if=/dev/block/bml4 of=/sdcard/Sbl.bin bs=4096'
$CMD shell su -c 'dd if=/dev/block/bml4 of=/sdcard/Sbl.bin bs=4096'
echo 'dd if=/dev/block/bml5 of=/sdcard/untested-recoverySbl.bin bs=4096'
$CMD shell su -c 'dd if=/dev/block/bml5 of=/sdcard/untested-recoverySbl.bin bs=4096'
echo 'dd if=/dev/block/bml6 of=/sdcard/param.lfs bs=4096'
$CMD shell su -c 'dd if=/dev/block/bml6 of=/sdcard/param.lfs bs=4096'
echo 'dd if=/dev/block/bml7 of=/sdcard/zImage bs=4096'
$CMD shell su -c 'dd if=/dev/block/bml7 of=/sdcard/zImage bs=4096'
echo 'dd if=/dev/block/bml8 of=/sdcard/untested-empty.bin bs=4096'
$CMD shell su -c 'dd if=/dev/block/bml8 of=/sdcard/untested-empty.bin bs=4096'
echo 'dd if=/dev/block/bml9 of=/sdcard/factoryfs.rfs bs=4096'
$CMD shell su -c 'dd if=/dev/block/bml9 of=/sdcard/factoryfs.rfs bs=4096'
echo 'dd if=/dev/block/bml10 of=/sdcard/dbdata.rfs bs=4096'
$CMD shell su -c 'dd if=/dev/block/bml10 of=/sdcard/dbdata.rfs bs=4096'
echo 'dd if=/dev/block/bml11 of=/sdcard/cache.rfs bs=4096'
$CMD shell su -c 'dd if=/dev/block/bml11 of=/sdcard/cache.rfs bs=4096'
echo 'dd if=/dev/block/bml12 of=/sdcard/modem.bin bs=4096'
$CMD shell su -c 'dd if=/dev/block/bml12 of=/sdcard/modem.bin bs=4096'
echo 'dd if=/dev/block/mmcblk0p2 of=/sdcard/data.bin bs=4096'
$CMD shell su -c 'dd if=/dev/block/mmcblk0p2 of=/sdcard/data.bin bs=4096'
# or if you have busybox
#$CMD shell su -c 'tar -czvf /sdcard/data.tar.gz /data'
echo
sleep 10

echo "Copying created images to: $DST"
echo
echo pull /sdcard/boot.bin "$DST"
$CMD pull /sdcard/boot.bin "$DST"
echo pull /sdcard/pit.pit "$DST"
$CMD pull /sdcard/pit.pit "$DST"
echo pull /sdcard/untested-efs.rfs "$DST"
$CMD pull /sdcard/untested-efs.rfs "$DST"
echo pull /sdcard/Sbl.bin "$DST"
$CMD pull /sdcard/Sbl.bin "$DST"
echo pull /sdcard/untested-recoverySbl.bin "$DST"
$CMD pull /sdcard/untested-recoverySbl.bin "$DST"
echo pull /sdcard/param.lfs "$DST"
$CMD pull /sdcard/param.lfs "$DST"
echo pull /sdcard/zImage "$DST"
$CMD pull /sdcard/zImage "$DST"
echo pull /sdcard/untested-empty.bin "$DST"
$CMD pull /sdcard/untested-empty.bin "$DST"
echo pull /sdcard/factoryfs.rfs "$DST"
$CMD pull /sdcard/factoryfs.rfs "$DST"
echo pull /sdcard/dbdata.rfs "$DST"
$CMD pull /sdcard/dbdata.rfs "$DST"
echo pull /sdcard/cache.rfs "$DST"
$CMD pull /sdcard/cache.rfs "$DST"
echo pull /sdcard/modem.bin "$DST"
$CMD pull /sdcard/modem.bin "$DST"
echo pull /sdcard/data.bin "$DST"
$CMD pull /sdcard/data.bin "$DST"
# or
#$CMD pull /sdcard/data.tar.gz "$DST"
echo
sleep 10

echo 'Removing images from internal SD card...'
echo
$CMD shell rm /sdcard/boot.bin
$CMD shell rm /sdcard/pit.pit
$CMD shell rm /sdcard/untested-efs.rfs
$CMD shell rm /sdcard/Sbl.bin
$CMD shell rm /sdcard/untested-recoverySbl.bin
$CMD shell rm /sdcard/param.lfs
$CMD shell rm /sdcard/zImage
$CMD shell rm /sdcard/untested-empty.bin
$CMD shell rm /sdcard/factoryfs.rfs
$CMD shell rm /sdcard/dbdata.rfs
$CMD shell rm /sdcard/cache.rfs
$CMD shell rm /sdcard/modem.bin
$CMD shell rm /sdcard/data.bin

echo 'Done.'

